C    A simple parallel program to solve u_t + u_x = 0 on periodic domain
C    from 0 to 1. With u(x, t = 0) = sin(2*pi*x)
C   
       subroutine worker()
       integer          N,nn,offset,i,M, win,nox
       double precision x(10000), u(10000), v(10000), err, work 
       double precision h, k, xstart, pi, lambda, tfinal
       parameter        ( pi = 3.14159265358979 )
#include "comm/fcomm.h"
#include "xtools/fbasex11.h"

       if (PImytid() .eq. 0) then
         print*, 'Enter N'
         read*, N
         print*, 'Enter 1 for X Windows or 0 for not'
         read*, nox
       endif
       Call PIbcastSrc(N , intsze, 0, PSAllProcs, MSG_INT)
       Call PIbcastSrc(nox , intsze, 0, PSAllProcs, MSG_INT)

       if (nox .eq. 1) then
         win    = XBOpenPerProcessorWindow('Solution',0)
       else
         win = 0
       endif 
       h      = 1.d0/N
       k      = h
       lambda = k/h
       M      = .5d0/k
       nn     = (N + PInumtids() - 1)/PInumtids()
       offset = nn*PImytid()
       if ( (nn + offset) .gt. N ) then
         nn = N - offset
       endif
       nn      = nn + 2
       xstart = h*(offset - 1)

       if (nn .gt. 1000) then
         print*,'Recompile wavef.F with a larger allocation'
         call SYexitall(0,0)
       endif
       
       x(1) = xstart
       do 10, i=2,nn
         x(i) = x(i-1) + h
 10    continue

       do 20 i=1,nn
         u(i) = dsin(2.d0*pi*x(i))
 20    continue
       if (win .ne. 0) call GraphSolution(win,x,u,nn)

       do 30 i=1,M
         call TimeStep(u,v,nn,lambda)
         call UpDateEndPoints(v, nn)
         if (win .ne. 0) call GraphSolution(win,x,v,nn)
         call TimeStep(v,u,nn,lambda)
         call UpDateEndPoints(u, nn)
         if (win .ne. 0) call GraphSolution(win,x,u,nn)
 30    continue

C   Determine Error at final time  
    
       tfinal = 2.d0*M*k
       err = 0.d0
       do 40 i=2,nn-1
         err = err + (dsin(2.d0*pi*(x(i) - tfinal)) - u(i))**2.d0
 40    continue
       call PIgdsum(err,1,work,PSAllProcs)
       err = dsqrt(err)
       if (PImytid() .eq. 0) then 
         print*,'Total points ',N, ' Two norm of error ',err
       endif 
       return
       end

       subroutine UpDateEndPoints(u,n)
       integer          n,left,right
       double precision u(n)
#include "comm/fcomm.h"

       right = PImytid() + 1
       if (right .eq. PInumtids()) right = 0
       left  = PImytid() - 1
       if (left .eq. -1) left = PInumtids() - 1

       call PIbsend(1,u(n-1), dblsze,right,MSG_DBL)
       call PIbrecv(1,u(1),   dblsze,MSG_DBL) 
       call PIbsend(2,u(2),   dblsze ,left,MSG_DBL)
       call PIbrecv(2,u(n),   dblsze,MSG_DBL)
       return
       end 
       
       subroutine TimeStep(u,v,n,lambda)
       integer n,i
       double precision u(n),v(n),lambda

       do 10 i=2,n-1
         v(i) = .5d0*( (u(i+1) + u(i-1)) - lambda*(u(i+1) - u(i-1)) )
 10    continue
       return 
       end

       subroutine GraphSolution(win,x,u,n)
       integer win, n, i, lg
       double precision x(*), u(*)
#include "xtools/fbasex11.h"
       lg = XBLineGraphCreate(win, 1)
       do 10 i=1,n
         call XBLineGraphAddPoint(lg,x(i),u(i))
 10    continue
       call XBLineGraphSetLimits(lg,x(1),x(n),-1.d0,1.d0)
       call XBLineGraphDraw(lg)
       call XBLineGraphDestroy(lg)
       return 
       end
