#ifndef lint
static char vcid[] = "$Id: gtoken.c,v 1.2 1994/06/16 20:23:07 gropp Exp $";
#endif

#include "comm/comm.h"
#include "comm/global/global.h"

static int _gtokdata = 0;

/*
   This is a special "token" routine.  A common use of GSYNC is:

   for (i=0;i<NUMNODES;i++) {if (i==MYPROCID) <do-something>; GSYNC(procset);}

   This is rather expensive when all we are doing is passing a handle around
   saying "you-can-go-next".  We replace this with

   for (i=0;i<=NUMNODES;i++) {if (gtoken(procset,i)) <do-something>;}

   Note that the loop must be executed one additional time.
 */
int gtokenset( procset, i )
ProcSet *procset;
int     i;
{
int right, left, root, myid, np;
int msgbase;

LOGOPSTART(LOG_GTOKS);
LOGPUSHATOMIC;
/* printf( "[%d] doing gtoken\n", MYPROCID ); */
if (!procset) {
    myid    = MYPROCID;
    np      = NUMNODES;
    left    = (myid == 0) ? np-1 : myid - 1;
    right   = myid + 1;
    if (right >= np) right = 0;
    root    = 0;
    }
else {
    myid  = procset->myid;
    np    = procset->nump;
    right = procset->right;
    left  = procset->left;
    root  = procset->root;
    }
msgbase = GMSGTYPE(procset,0);

/* These aren't quite right for the general case (need the relative index 
   of MYPROCID in the ring/mesh */
if (i == myid && myid != root) {
    RECVSYNCNOMEM(msgbase|left,&_gtokdata,sizeof(int),MSG_INT);
    }
else if ((i-1) == myid) {
    SENDSYNCNOMEM(msgbase|myid,&_gtokdata,sizeof(int),right,MSG_INT);
    }
else if (myid == root && i == np) {
    RECVSYNCNOMEM(msgbase|left,&_gtokdata,sizeof(int),MSG_INT);
    }
LOGOPEND(LOG_GTOKE);
LOGPOPATOMIC;
return (i == myid);
}

/*
    These two routines allow the user to get/set the token that is
    passed around
 */
void PISetToken( val )
int val;
{
_gtokdata = val;
}

int PIGetToken()
{
return _gtokdata;
}	

/*@
   PISeqBegin - Begin a sequential section of code.  

   Input Parameters:
.  procset - Processor set to sequentialize.  
.  ng - Number in group.  This many processes are allowed to execute
   at the same time.  Usually one.  

   Notes:
   PISeqBegin and PISeqEnd provide a way to force a section of code to
   be executed by the processes in rank order.  Typically, this is done 
   with
$  PISeqBegin( PSAllNodes, 1 );
$  <code to be executed sequentially>
$  PISeqEnd( PSAllNodes, 1 );
$
   Often, the sequential code contains output statements (e.g., printf) to
   be executed.  Note that you may need to flush the I/O buffers before
   calling PISeqEnd; also note that some systems do not propagate I/O in any
   order to the controling terminal (in other words, even if you flush the
   output, you may not get the data in the order that you want).
@*/
void PISeqBegin( procset, ng )
ProcSet *procset;
int     ng;
{
int msgbase, lidx, token;

msgbase = GMSGTYPE(procset,0);
lidx    = PSmytid(procset);
if (lidx != 0) {
    PIbrecv( msgbase, &token, sizeof(int), MSG_INT );
    }
/* Send to the next process in the group unless we are the last process 
   in the processor set */
if ( (lidx % ng) < ng - 1 && lidx != PSnumtids(procset) - 1) {
    PIbsend( msgbase, &token, sizeof(int), PStidFromRank( procset, lidx + 1 ), 
	     MSG_INT );
    }
}

/*@
   PISeqEnd - End a sequential section of code.

   Input Parameters:
.  procset - Processor set to sequentialize.  
.  ng - Number in group.  This many processes are allowed to execute
   at the same time.  Usually one.  

   Notes:
   See PISeqBegin for more details.
@*/
void PISeqEnd( procset, ng )
ProcSet *procset;
int     ng;
{
int msgbase, lidx = PSmytid(procset), np = PSnumtids(procset);
int token;

msgbase = GMSGTYPE(procset,0);
/* Send to the first process in the next group OR to the first process
   in the processor set */
if ( (lidx % ng) == ng - 1 || lidx == np - 1) {
    PIbsend( msgbase, &token, sizeof(int), 
             PStidFromRank( procset, (lidx + 1) % np ), 
	     MSG_INT );
    }
if (lidx == 0) {
    PIbrecv( msgbase, &token, sizeof(int), MSG_INT );
    }
}


