#ifndef lint
static char vcidI[] = "$Id: initpvm3.c,v 1.9 1994/12/15 20:38:13 gropp Exp $";
#endif

/*

  This is a pvm3 version.  It is derived from the pvm2 version; there may be
  some parts of this code that are derived from work arounds to problems in
  pvm2 that may have been removed in pvm3.
  
     There may be a problem with the way this is intended to run.
  The main process sets a link for the executable then immediately tells
  the other processor (with an initiate()) to run it. It seems that the
  other machine may look for the link and not find it, because it
  has been cached somehow. We need a way of flushing over the entire
  NFS network of any directory caches.

     I put a sleep() in but that is not very satisfactory. 

*/

/*  PVM has many bad design decisions; we should write them down and 
grind our teeth. But I don't want to change it at all; we must write
an interface so we can use standard PVM without changing our upper level
code.

  1) Limited path for looking for executables.
  2) Does not set a reasonable working directory.
  3) Does not pass command line.
  4) There is no routine to return the number of machines of 
     a particular architecture. (pvm_config now?)
  5) stderr and stdout go to where the deamons are started up.
  6) makes no attempt to catch interupts on user process; one could
     die and the PVM system wouldn't even know. (still true?)
  7) in unpacking messages user must know exactly how they were packed.
  8) Must have a pvm daemon running that has the number of processors
     you want.  You can't simply generate a list of processors to run
     on (it really is a virtual machine, defined statically).  We'll
     fix this by starting one with the list of workstations to use.

  In order to handle (1), we need to establish links to the appropriate 
  machines (PVMCreateLink).  2-3 are handled by PIcall startup code.
  There isn't much that we can do about 5; 6 we attempt to handle by
  catching the interrupts.  7 is a feature.  4 and 8 go together, and
  are tricky to deal with.  

  Since PVM won't tell us what machines are available, we may need the
  name of the file that started the pvmd.  If we don't have that, we
  have to fake it.

  Ah, just discovered that PVM 2.1.4 WILL give you the machines.  The 
  code is:
  pvm_mstat( int *nhosts, int **ifprocs, char ***hosts, char ***arches )
  (see Skjellum's paper, page 7)

  pvm 3 changes this routine without changing the name (!)
*/

#define MAXFILELEN 31 /* limit of PVM */
#define MAXPVMCOMPLEN 9 /* limit of PVM */

/* Define CANSTARTDAEMON to allow PICall to try and start a PVM daemon */
#define CANSTARTDAEMON

#include "comm/comm.h"
#include "comm/hosts.h"
#include "system/system.h"
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#if defined(rs6000) || defined(solaris) || defined(HPUX) || defined(intelnx)
#include <signal.h>
#endif

#if defined(intelnx)
typedef u_short uid_t;
typedef u_short gid_t;
#endif

#include "pvm3.h"

void PVMCreateLink();
void PVMRemoveLinks();
void PVMArchType();
void PVMexitall();
extern char *getenv();

/* This defines these values */
int  __PVMFROM, __PVMFROMTID, __PVMLEN, __PVMTYPE, __PVMGLOBALTYPE;
int  __MYPROCID, __NUMNODES;

/* static char *PVMGroup = 0; */
/* PVMtids carries the mapping from 0:np-1 to tids returned by pvm_spawn */
#ifdef T9000
int  *PVMtids  = 0;
#else
static int  *PVMtids  = 0;
#endif

static int __PVMSpawnFlag = 0;

/* #define CpvmMapToTid( pid )   pvm_gettid( PVMGroup, pid ) */
/* #define CpvmMapFromTid( tid ) pvm_getinstr( PVMGroup, tid ) */

#define CpvmMapToTid( pid )   PVMtids[pid]
/* We need a routine to match sender to rank.  This could be expensive, 
   so we should defer it */

static int pvmdpid = 0;
static int pvm_use_vsnd = 0;
#if defined(LOGCOMMDISABLE)
#define LOGpush()
#define LOGpop()
#endif

/* 
   PIcall - version for PVM. It uses argv[0] to determine the 
            name of the other np-1 instances of the  process group. 
   
  Things still needed to be done for this interface.

   1) Check if pvmd is running; if not create hosttable and start it up

   ??? HeNCE

*/
#define CBUFLEN 10

int PIiFinish()
{
  PIendlog();
  GSYNC((ProcSet*)0);
  pvm_exit();
  if (MYPROCID == 0) {
      PVMRemoveLinks();
      /* Kill off the pvm daemon if we started it */
#ifdef SIGINT
      if (pvmdpid)
	  kill( pvmdpid, SIGINT );
#endif
      }
return 0;
}

int PIiInit( argc, argv)
int  *argc;
char ***argv;
{
  HostTable *table, *utable;
  int       i,j,k,l,count,len,fsec, maxarches = MAXARCHES, rc, np;
  char      cwd[MAXPATHLEN],arch[20], bname[MAXFILELEN], *machine;
  char      fname[MAXPATHLEN], cbuf[CBUFLEN];
  char      *archpath,*path, hostfile[MAXPATHLEN], oarch[20];
  Arch      narch,iarch, tid;
  FILE      *fd;

  /* setup to catch interrupts; include ^C */
  SYSetDefaultSignals( SYDefaultHandler );
#ifdef SIGINT
  signal( SIGINT, SYDefaultHandler ); 
#endif

  /* set the exit routine */
  SYSetExitAll(PVMexitall);

  /* determine base name of our, to be created, process group */
  PIBaseName( (*argv)[0],0,bname, MAXFILELEN );
/*   strncpy(__PVMGroup,bname,MAXPVMCOMPLEN);
  __PVMGroup[MAXPVMCOMPLEN-1] = 0; 
 */
  /* Note: Barry reports that names of more than eight
     characters cause problems. */ 

  tid = pvm_parent();
  if (tid == PvmSysErr) {
      /* Could not contact pvmd; consider starting one up */
      fprintf( stderr, "Could not find a PVM to use; please start one\n" );
      exit(1);
      }
  else if (tid == PvmNoParent) {
      /* We are the master */
      __MYPROCID = 0;
      }
  else 
      __MYPROCID = 1;  /* Place holder */
  /* enroll in pvm.  */
  tid = pvm_mytid();

  /* One problem with PVM is there is no way to determine if we are
     the 0th or nth process.  This can be a problem if there are some
     hung processes and the application is restarted.  Then, the
     value from enroll, which we are COUNTING on being 0, may be
     > 0, since the hung processes will be counted.  There is no
     certain way to handle this, but we can take advantage of the fact
     that PVM does not pass command line arguments when processes
     are started.  Thus, if argc > 1 and __MYPROCID > 0, there is a
     problem. */

  /* I'll wait until I know how to detect a missing pvmd to think about
     starting a pvmd up if none is running.  See initpvm.c (the 2.4.x version)
     for code */

  /* determine our arch */
  SYGetArchType(arch,20); narch = PIStringToArch(arch);

  LOGpush(); LOGDISABLE;

  /* Check for pvm-specific options */
  if (SYArgGetString( argc, (*argv), 1,"-pvm", cbuf, CBUFLEN ))
      PISetOption( "pvm", cbuf, (void *)0 );

  /* Set the default np */
  np = 1;
  if (MYPROCID == 0) {
      /* we are the host process; */
      /* determine if we have enough processors */
      SYArgGetInt( argc, (*argv), 1, "-np", &np );
      /* 
	Since we can start multiple tasks on the same node, this test
	is not accurate.  We should let the host table routines handle this
       */
#ifdef CHECK_ON_PROCESSES
      { int narch;
      pvm_config( &__NUMNODES, &narch, (struct hostinfo **)0 );
      }
      if (__NUMNODES < np) { 
	  fprintf(stderr,
		  "Not enough processors: want %d total %d \n",np,__NUMNODES);
	  SETERR(2); pvm_exit(); 
#ifdef SIGINT
	  if (pvmdpid) kill( pvmdpid, SIGINT); return -1;
#endif
	  }
#endif
      __NUMNODES = np--; 
      /* np is now the number left to start */
      PVMtids = (int *)MALLOC( (np + 1) * sizeof(int) );  CHKPTR(PVMtids);
      PVMtids[0] = tid;
      
      /* Get resource limits */
      SYGetResourceDefaults( &cpu, &mem, &pf, &niceval, &fsec );
      SYArgGetInt( argc, (*argv), 1, "-nice", &niceval );
      gettimeofday( &maxtime, 0 );  maxtime.tv_sec += fsec;
      SYGetResourceLimits( argc, (*argv), &cpu, &mem, &pf, &maxtime, 1 );
      
      SYGetwd( cwd, MAXPATHLEN );
      
      /* Note that this should really be generated from the file that 
	 started the pvmd */ 
      utable = 0;
      table  = 0;
      if (!pvmdpid && PIiPVMFormHostTable( &utable, &table, 
					  np, mem, cpu, pf, arch, bname, 
					  cwd, argc, argv ) == -1) { 
	  pvm_exit(); return -1; }
      if (!pvmdpid && !utable) {pvm_exit(); 
				SETERRC(3,"Could not get host table"); 
				return -1;}
      
      if (pvmdpid || !utable) {
	  if (!PIFullPathFromBase(bname,-1,(char *)0,path,fname,
				  MAXPATHLEN,cwd)) 
	      if (!PIFullPathFromBase(bname,arch,(char *)0,path,fname,
				      MAXPATHLEN,cwd))
		  strcpy(fname,bname);
	  }
      if (PIiPVMCreateLinks( utable, maxarches, bname, arch, fname ) == -1) 
	  return -1;
      if (PIiPVMStartWorkers( utable, maxarches, bname, np, fname ) == -1) 
	  return -1;
      if (utable) PIDestroyHostTable(utable);
      if (table)  PIDestroyHostTable( table);
      /* Send the number of processors and their tids */
      for ( i=0; i<np; i++ ) {
	  SENDSYNC(10001,&__NUMNODES,sizeof(int),i+1,MSG_INT);
	  SENDSYNC(10002,PVMtids,__NUMNODES*sizeof(int),i+1,MSG_INT );
	  }
      }
  else {
      /* receive number of processors */
      RECVSYNC(10001,&__NUMNODES,sizeof(int),MSG_INT);
      PVMtids = (int *)MALLOC( __NUMNODES * sizeof(int) );  CHKPTR(PVMtids);
      RECVSYNC(10002,PVMtids,__NUMNODES*sizeof(int),MSG_INT );
      /* If we are over the limit of number of nodes, we should exit here ? */
      for ( i=0; i<__NUMNODES; i++) {
	  if (PVMtids[i] == tid ) __MYPROCID = i;
	  }
      }
  
  PISetupCollectiveTree( );
  /* receive resource limits */
  PIBroadcastLimits( &cpu, &mem, &pf, &maxtime, &niceval, 
		    MYPROCID==0,(ProcSet *)0 );
  SYSetResourceLimits( mem, cpu, pf, &maxtime );
  if (niceval > 0) SYSetNice( niceval );
  
  /* receive the command line */
  PIBroadcastArgs( argc, argv, MYPROCID==0, (ProcSet *)0 ); 
  PIGetDebugArgs( argc, *argv, MYPROCID == 0 );
  
  /* get the current working directory and chdir to it*/
#ifndef T9000
  len = strlen(cwd);
#else
  len = strlen(cwd)+1;
#endif
  GSCATTER(&len, sizeof(int), MYPROCID==0, (ProcSet *)0, MSG_INT);
  GSCATTER(cwd, len, MYPROCID==0, (ProcSet *)0, MSG_OTHER);
  chdir(cwd);
  
  /* pass to user code */
  LOGpop();
  PIinitlog();
  /* This GSYNC is used to insure that all processors reach this point
     (bad things can happen if someone does a SYexitall before all
     tasks reach this point) */
  GSYNC((ProcSet*)0);
  PISummaryClear();
return 0;
}

int PIcall(np, procgroup, dummy, routine, argc, argv)
int  np, argc;
char **procgroup, *dummy;
char **argv;
int  (*routine)();
{
  int rc;
  if (rc = PIiInit( &argc, &argv ))  return rc;
  rc = (*routine)(argc,argv);
  PIiFinish();
  return rc;
}

/*
    exitall - terminate all processes in an execution (parallel exit)
    See init.c for a complete description of the interface.
 */
void PVMexitall(msg,rc)
char *msg;
int  rc;
{
   int i;
   if (msg) 
       fputs( msg, stderr );
   for ( i=0; i<NUMNODES; i++ ) {
     if (i != MYPROCID) pvm_kill( CpvmMapToTid( i ) );
   }
   if (!MYPROCID) PVMRemoveLinks();
   pvm_exit();
   fflush(stdout);
   fflush(stderr);
   exit(1);
}

/*
 Here are pvm support routines.  These take datatype as an argument,
 and convert it into a routine name.  An alternate approach is to
 have an array of routines, and use the data_type to dereference the
 array
 */
/* 
   Note:  When sending a byte stream, it first prepends an integer length 
  of the byte stream; then puts down the bytes, then pads so the 
  total length is divisible by four; therefore there is no way to 
  to check whether the arriving message is the expected size. 
 
    We should make sure that we never use MSG_OTHER except for 
  character strings; not even for structures on the same architecture.
*/

Cpvm_send( type, to, buffer, length, datatype )
int  type, to, length, datatype;
char *buffer;
{
int bufid;	
#if defined(rs6000) 
SYSetResourceClockOff();
#endif

bufid = pvm_initsend( PvmDataDefault ) ;  /* PvmDataDefault == 0 */
/* Note the use of PvmDataInPlace (== 2) for removing a possible copy.  But
   note that you can't combine the two! */
if (length) {
  switch (datatype) {
    case MSG_LNG:   length = length / sizeof(long);
		    pvm_pklong( (long *)buffer, length, 1 ); break;
    case MSG_INT:   length = length / sizeof(int);
		    pvm_pkint( (int *)buffer, length, 1 ); break;
    case MSG_FLT:   length = length / sizeof(float);
		    pvm_pkfloat( (float *)buffer, length, 1 ); break;
    case MSG_DBL:   length = length / sizeof(double);
		    pvm_pkdouble( (double *)buffer, length, 1 ); break;
    default:
    case MSG_OTHER: pvm_pkbyte( (char *)buffer, length, 1 ); break;
    }
  }
pvm_send( CpvmMapToTid( to ), type );

#if defined(rs6000)  
SYSetResourceClockOn();
#endif
}

Cpvm_recv( type, from, buffer, length, datatype )
int  *type, *length, *from, datatype;
char *buffer;
{
int  bytes, msgtype;
char *component;
int  tid, bufid;

#if defined(rs6000) 
SYSetResourceClockOff();
#endif

bufid = pvm_recv( -1, *type );

pvm_bufinfo( bufid, &bytes, &msgtype, &tid );
__PVMFROMTID = tid;
__PVMFROM    = -1;
/* If user wants from, they'll have to convert it with a routine not
   yet defined */
*type = msgtype;
/* We may want to check that the component name matches */

if (bytes > *length) {
  bytes = *length;
  /* Error condition - truncate */
  /* ? use pvm error routine ? */
  if ( datatype != MSG_OTHER ) {  /* see note above */
    fprintf(stderr,"[%d] received message too long from %d, %d %d truncating\n"
                , MYPROCID,*from,bytes,*length);
    }
  }

*length = bytes;

if (bytes) {
  switch (datatype) {
    case MSG_LNG:   bytes = bytes / sizeof(long);
	            pvm_upklong( (long *)buffer, bytes, 1 ); break;
    case MSG_INT:   bytes = bytes / sizeof(int);
		    pvm_upkint( (int *)buffer, bytes, 1 ); break;
    case MSG_FLT:   bytes = bytes / sizeof(float);
 		    pvm_upkfloat( (float *)buffer, bytes, 1 ); break;
    case MSG_DBL:   bytes = bytes / sizeof(double);
		    pvm_upkdouble( (double *)buffer, bytes, 1 ); break;
    default:
    case MSG_OTHER: pvm_upkbyte( (char *)buffer, bytes, 1 ); break;
    }
  }
#if defined(rs6000)  
SYSetResourceClockOn();
#endif
}

/* Receive a message of unknown size */
Cpvm_recv_unsz( type, from, buffer, length, datatype )
int  *type, *from, *length, datatype;
char **buffer;
{
int  bytes, msgtype;
int tid, bufid;

#if defined(rs6000) 
SYSetResourceClockOff();
#endif

fprintf(stderr, "Cpvm_recv_unsz\n" );
bufid = pvm_recv( -1, *type );
pvm_bufinfo( bufid, &bytes, &msgtype, &tid );
*length = bytes; *type = msgtype;
*from   = CpvmMapFromTid( tid );
*buffer = MALLOC(bytes); CHKPTR(*buffer);

if (bytes) {
  switch (datatype) {
    case MSG_LNG:   bytes = bytes / sizeof(long);
	            pvm_upklong( (long *)*buffer, bytes, 1 ); break;
    case MSG_INT:   bytes = bytes / sizeof(int);
		    pvm_upkint( (int *)*buffer, bytes, 1 ); break;
    case MSG_FLT:   bytes = bytes / sizeof(float);
 		    pvm_upkfloat( (float *)*buffer, bytes, 1 ); break;
    case MSG_DBL:   bytes = bytes / sizeof(double);
		    pvm_upkdouble( (double *)buffer, bytes, 1 ); break;
    default:
    case MSG_OTHER: pvm_upkbyte( (char *)*buffer, bytes, 1 ); break;
    }
  }
#if defined(rs6000)  
SYSetResourceClockOn();
#endif
}
 
#define PI_HAS_INIT
#define PI_HAS_EXIT

/*   Given a path of executable creates a soft link to                 */
/*   $HOME/pvm/bin/$ARCH                                               */
/*  so that the pvmd can find and load it.                             */
/*  1) make sure pname exists.                                         */
/*  2) make sure $HOME/pvm3/bin/$ARCH exists; if not create it.        */
/*     NOTE: pvm uses different $ARCHes then we do                     */
/*  3) create link.                                                    */
/*
/*      iarch - arch of machine making link for                        */
/*      name - name of file being linked                               */
/*      bname - basename of new file created                           */

int  __PVMNumberLinks = 0;
char *__PVMLinks[MAXARCHES];

void PVMCreateLink(iarch,name,bname)
char *name,*bname;
Arch iarch;
{
  char   truename[MAXPATHLEN], pname[MAXPATHLEN], tmppath[MAXPATHLEN];
  char   arch[20], homedir[MAXPATHLEN];
  struct passwd *pwde;
  int    err,len;
  struct stat statbuf;

  PIArchToString(arch,iarch); PVMArchType(arch,20);

  /* get true path */
  if (!SYGetRealpath(name,truename)) {
      SETERRC(5,"Could not get executable's path"); pvm_exit(); return;}

  /* determine home directory */
  pwde = getpwuid( geteuid() );  
  if (!pwde) {
      SETERRC(6,"Could not get user's home directory"); pvm_exit(); return;}
  strcpy( tmppath, pwde->pw_dir );
#if defined(rs6000) || defined(T9000)
  strcpy(homedir,tmppath);
#else
  if (!realpath(tmppath,homedir)) {
      SETERRC(5,"Could not get user's home directory"); pvm_exit(); return;}
#endif

  /* make sure subdirectories are in place */
  strcat(homedir,"/pvm3");

  err = stat( homedir, &statbuf ); 
  if (err) {
    if (mkdir(homedir,S_IRUSR|S_IWUSR|S_IXUSR)) {
        SETERRC(8,"Could not make pvm3 executable directory");
	pvm_exit();return;} 
  }
  strcat(homedir,"/bin");

  err = stat( homedir, &statbuf ); 
  if (err) {
    if (mkdir(homedir,S_IRUSR|S_IWUSR|S_IXUSR)) {
        SETERRC(8,"Could not make pvm3/bin executable directory");
	pvm_exit();return;} 
  }

  strcat(homedir,"/");
  strcat(homedir,arch);  
  err = stat( homedir, &statbuf ); 
  if (err) {
    if (mkdir(homedir,S_IRUSR|S_IWUSR|S_IXUSR)) {
        SETERRC(8,"Could not make pvm3/bin/ARCH executable directory");
	pvm_exit();return;} 
  }
  strcat(homedir,"/");
  strcat(homedir,bname);

  /* is the file already there ? */
  err = stat( homedir, &statbuf );
  if (err) {
    if (symlink(truename,homedir)) {
        SETERRC(9,"Could not link to pvm executable");pvm_exit();return;}
    len = strlen(homedir);
    __PVMLinks[__PVMNumberLinks] = (char *) MALLOC( (len+1)*sizeof(char) );
    if (!__PVMLinks[__PVMNumberLinks]) {
        SETERRC(10,"Could not allocate space for file links");pvm_exit();return;}
    strcpy(__PVMLinks[__PVMNumberLinks++],homedir);
  }
}
/* 
      Removes any links set by PVMCreateLink().
*/
void PVMRemoveLinks()
{
  int i;
  for ( i=0; i<__PVMNumberLinks; i++ ) {
    unlink(__PVMLinks[i]);
  }
}

/*        
     Given out string name of an architecture; returns PVM name in the
   same location.  len should be at least 5 (NOT 4!).
*/
  
void PVMArchType(arch,len)
char *arch;
int  len;
{
  if (!strncmp(arch,"sun4",4))    { strncpy(arch,"SUN4",len); return; }
  if (!strncmp(arch,"sun3",4))    { strncpy(arch,"SUN3",len); return; }
  if (!strncmp(arch,"tc2000",6))  { strncpy(arch,"BFLY",len); return; }
  if (!strncmp(arch,"intelnx",7)) { strncpy(arch,"I860",len); return; }
  if (!strncmp(arch,"NeXT",4))    { strncpy(arch,"NEXT",len); return; }
  if (!strncmp(arch,"rs6000",6))  { strncpy(arch,"RIOS",len); return; }
  if (!strncmp(arch,"IRIX",4))    { strncpy(arch,"SGI",len); return; }
  if (!strncmp(arch,"hpux",4))    { strncpy(arch,"HPPA",len); return; }
  if (!strncmp(arch,"T9000",5))    { strncpy(arch,"T9000",len); return; }

}

/* This routine forms the host table to give to start a pvm daemon, OR
   determine the hosts to select. Returns -1 on failure, 0 on success */
int PIiPVMFormHostTable( utable, table, np, mem, cpu, pf, arch, bname, cwd, 
			 Argc, Argv )
HostTable **utable, **table;
int       np, mem, cpu, pf;
char      *arch, *bname, *cwd;
int       *Argc;
char      ***Argv;
{
int       maxarches = MAXARCHES, i;
char      *archpath,*path, hostfile[MAXPATHLEN];
Arch      narch;
FILE      *fd;

/* get environmental variables needed  */
SYGetwd( cwd, MAXPATHLEN );
archpath = getenv("TOOLSARCHES");if (!archpath) archpath = arch;
path     = getenv("PATH"); if (!path) path = cwd;
narch    = PIStringToArch( arch );

/* Get database of available processors */
SYGetFileFromEnvironment( "TOOLSPVMHOSTS", DEFAULTPVMHOSTS,
			 "hosts", hostfile, 'r' ); 
fd = fopen(hostfile,"r"); 
if (!fd) {
    fprintf(stderr,"Could not open hosts database %s \n",hostfile);
    fprintf( stderr, "Set TOOLSPVMHOSTS\n" );
    SETERR(1); return -1;
    }
*table = PIReadInHostTable( fd, mem, cpu, pf, 0 ); fclose(fd); 
if (!*table) {SETERRC(2,"Could not read in host table"); return -1;}

*utable = 
    PIBuildHostTable(bname,np,*table,Argc,Argv,path,archpath,0,narch,cwd,0); 
if (!*utable) {SETERRC(3,"Could not build host table"); return -1;}
return 0;
}

/* This routine generates the links to the executables */
int PIiPVMCreateLinks( utable, maxarches, bname, arch, fname )
HostTable *utable;
int       maxarches;
char      *bname, *arch, *fname;
{
int  i;

if (!utable) 
    PVMCreateLink( 0, fname, bname );
else {
    for ( i=0; i<maxarches; i++ ) {
	if (utable->archtable[i].np) {
	    /* this bit of code is to get around a stupid feature of PVM */
	    /* makes a link of executable to the users $HOME/pvm3/bin/$ARCH */
	    PVMCreateLink(i,utable->archtable[i].fname,bname); CHKERRV(5,-1);
	    }
	}
    }
sleep(3); /* this sleep is to allow the directory caches to sync */
}

/* Actually start the processes */
PIiPVMStartWorkers( utable, maxarches, bname, np, fname )
HostTable *utable;
int       maxarches, np;
char      *bname, *fname;
{
int  count, i, j, k, l;
char *machine;

/* If we don't have a host table, just try to initialize processes */
if (!utable) {
    for ( k=0; k<np; k++ ) {
	/* There may be a problem if strlen(bname) > 8 */
#ifdef T9000_
	if (pvm_spawn( fname, (char **)0, PvmTaskHost | __PVMSpawnFlag, 
		       (char *)0, 1, PVMtids+1+k ) < 0) 
#else
	if (pvm_spawn( bname, (char **)0, PvmTaskHost | __PVMSpawnFlag, 
		       (char *)0, 1, PVMtids+1+k ) < 0) 
#endif
	{
	    fprintf(stderr,"Cannot initiate worker process %d\n", k);
	    fprintf(stderr,"base name %s program %s \n", bname, fname);
	    for ( l=0; l<k; l++ ) {
		if (l != MYPROCID) pvm_kill( PVMtids[l] );
		}   
	    PVMRemoveLinks();
	    pvm_exit();
	    SETERR(3); 
	    return -1;
	    }
	}
    return 0;
    }

count = 0;
/* loop over the arches in utable starting up the processes */
for ( i=0; i<maxarches; i++ ) {
    if (utable->archtable[i].np) {
	for ( j=0; j<utable->archtable[i].ne; j++ ) {
	    machine = utable->archtable[i].hosts[j]->name;
	    for ( k=0; k<utable->archtable[i].hosts[j]->np; k++ ) {
		/* pvm_spawn is SUPPOSED to return a value different from 
		   ntask when it fails, in which case the tids[] has
		   the error value.  Naturally, I found that only 
		   by testing PVMtids[] could I detect an error ... */
#ifdef T9000_
		if (pvm_spawn( utable->archtable[i].fname, (char **)0, 
#else
		if (pvm_spawn( bname, (char **)0, 
#endif
			       PvmTaskHost | __PVMSpawnFlag, machine, 1, 
			       PVMtids + 1 + count ) != 1 || 
		    PVMtids[count+1] < 0) {
		    fprintf(stderr,"Cannot initiate worker process %d %s \n",
			    i,machine);
		    fprintf(stderr,"bname %s program %s \n", 
			    bname,utable->archtable[i].fname);
		    for ( l=0; l<count; l++ ) {
			if (l != MYPROCID) pvm_kill( PVMtids[l] );
			}   
		    /* PVMtids[count+1] has error code */
		    PVMRemoveLinks();
		    pvm_perror("");
		    pvm_exit();
		    SETERR(3); 
		    return -1;
		    } 
		count++;
		}
	    }         
	}
    }
return 0;
}

#ifdef CANSTARTDAEMON
/* 
   This routine starts a default PVMD.  The default location of the
   daemon and the hosts file are given by
   DEFAULTPVMD and DEFAULTPVMHOSTS ; 
   these may be overridden with the ENVIRONMENT PARAMETERS
   
   TOOLSPVMD
   TOOLSPVMHOSTS

   The format of the file is 
   
   hostname dx=absolute-path-of-daemon-for-that-architecture
 */
int PIiPVMStartPVMD( np )
int np;
{
char pvmdfile[MAXPATHLEN];
char pvmhosts[MAXPATHLEN];
int  err;

SYGetFileFromEnvironment( "TOOLSPVMD", DEFAULTPVMD,
                            "/usr/tmp/pvmd", pvmdfile, 'r' ); 
/* We should actually be prepared to write the pvmhosts file, using the
   host-table code */
SYGetFileFromEnvironment( "TOOLSPVMHOSTS", DEFAULTPVMHOSTS,
                            "/usr/tmp/pvmhosts", pvmhosts, 'r' ); 
#ifdef T9000
	{
	char tmp[MAXPATHLEN*2+3];
	pvmdpid = 1;
	sprintf( tmp, "%s %s", pvmdfile, pvmhosts );
	system( tmp );
	}
#else
pvmdpid = fork();
if (pvmdpid == -1) {
    fprintf( stderr, "Could not start pvm daemon\n" );
    return -1;
    }
if (!pvmdpid) {
    /* Child */
    err = execl( pvmdfile, pvmdfile, pvmhosts, (char *)0 );
    }
else {
    /* Wait for the thing to start */
    fprintf( stdout, "Waiting for pvmd to start...\n" );
    sleep(10);
    }
#endif
return 0;
}
#endif


#define PI_HAS_SET_OPTION
void PISetOption( version, name, val )
char *version, *name;
void *val;
{
if (strcmp( version, "pvm" ) != 0) return;

if (strcmp( name, "list" ) == 0) {
    fprintf( (FILE*)val, "list - generate this list\n" );
    fprintf( (FILE*)val, 
"debug - Set PvmTaskDebug switch in spawn; should generate debug windows\n" );
    return;
    }

if (strcmp( name, "debug" ) == 0) {
    /* Allow the user to invoke PVM with debugger windows: */
    __PVMSpawnFlag |= PvmTaskDebug;
    }    
/* Eventually should add busywait */
}

/* Simple linear search for the tid to rank mapping.  A better system 
   could use a hash table on the tids, particularly if we knew more
   about the likely distribution of the tid values 
 */
int CpvmMapFromTid( tid )
int tid;
{
int i;
for (i=0; i<__NUMNODES; i++) {
    if (PVMtids[i] == tid) return i;
    }
return -1;
}


int Cpvm_probe( tag )
int tag;
{
fprintf( stderr, "PVM3.0 does not support a real probe.\n\
Consider using PIbrecvUnsz or special casing your code for pvm3.\n\
\n\
Alternately, if you have a later version of PVM that has pvm_probe,\n\
make sure that HAVE_PVM_PROBE is defined when compiling.\n" );
return 1;
}

#ifdef T9000
int  symlink( char *name1, char *name2 )
{
	char tmp[MAXPATHLEN*2+10];
	sprintf( tmp, "ln -s %s %s", name1, name2 );
	system(tmp);
	return(0);
}



#endif
