#ifndef lint
static char vcid[] = "$Id$";
#endif

#ifdef HAS_UNITREE
#include <stdio.h>
#include <ctype.h>
#if defined(cray)
#include <errno.h>
#else
extern int errno;
#endif
#include <sys/errno.h>
#include "tools.h"
#include "comm/comm.h"
#include "comm/io/pio.h"
#ifndef SEEK_SET
#include <unistd.h>
#endif

/* #if !defined(rs6000) */
extern int lseek();
/* #endif */

extern int read(), write();

/* 
   Default read, write, and seek routines.  These are simple versions
   of read, write, and lseek, but they take a PIFILE instead of an fd.
   This allows us to replace these routines with cached varieties.

   They ALSO handle EINTR

   These versions are specific to UNITREE
 */
int PIiDefUnitreeRead( fp, buffer, len )
PIFILE *fp;
char   *buffer;
int    len;
{
int err;

while (1) {
    /* RPC/POE do not coexist well.  We toggle SIGALRM around the nslread */
    alarm(0);
    err = nslread( fp->fd, buffer, len );
    ualarm(400000);
    /* printf("read, err = %d, errno = %d\n",err,errno); 
    fflush(stdout); */
    if (err < 0 && errno == EINTR) continue;
    if (err < 0) {
	SETERRC( 1, "Error in reading from file" );
	perror( (char *)0 );
	return -1;
	}
    break;
    }
return err;
}

int PIiDefUnitreeWrite( fp, buffer, len )
PIFILE *fp;
char   *buffer;
int    len;
{
int err;

while (1) {
    /* RPC/POE do not coexist well.  We toggle SIGALRM around the nslwrite */
    alarm(0);
    err = nslwrite( fp->fd, buffer, len );
    ualarm(400000);
    if (err < 0 && errno == EINTR) continue;
    if (err != len) {
	if (errno == ENOSPC) {
	    SETERRC( 1, "Error in writing to file; no space left" );
	    }
	else {
	    SETERRC( 1, "Error in writing to file" );
	    perror( (char *)0 );
	    }
	return -1;
	}
    break;
    }
return err;
}
int PIiDefUnitreeFlush( fp )
PIFILE *fp;
{
return 0;
}
int PIiDefUnitreeLseek( fp, offset, whence, rw )
PIFILE *fp;
int    offset, whence, rw;
{
int err;

/* RPC/POE do not coexist well.  We toggle SIGALRM around the nslread */
alarm(0);
err = nsllseek( fp->fd, offset, whence );
ualarm(400000);
if (err == -1) {
    perror( "Error performing lseek\n" );
    }
}

int PIiDefUnitreeClose( fp )
PIFILE *fp;
{
nslclose( fp->fd );
}

int PIiDefUnitreeOpen( fp, name, mode, fmode )
PIFILE *fp;
char   *name;
int    mode, fmode;
{
char tmpname[1024], *fname;

fname = name;
if (strncmp( name, "unitree:", 8 ) == 0) {
    strcpy( tmpname, name+8 );
    fname = tmpname;
    }

fp->fd = nslopen( fname, mode, fmode );
if (fp->fd < 0) {
    fprintf( stderr, "Error opening file %s with mode %x %x\n", 
	     fname, mode, fmode );
    perror( "Error opening file" );
    }
}

#ifndef PIF_FILEPREFIXDEF
#define PIF_FILEPREFIXDEF "/unihome/%u/"
#endif
/*+
   PIiDefUnitreeOpenEach - Open a parallel file 

   The name PIF_FILEPREFIX is an environment variable that may be used to
   select the file prefix.  By default, it is /???/%u/.  Intel systems may
   be compiled with a different default (define PIF_FILEPREFIXDEF). 
   %u is the user's login name and %n is the node number. 
+*/
int PIiDefUnitreeOpenEach( name, procset, mode, pmode, fmode )
char    *name;
ProcSet *procset;
int     mode, pmode, fmode;
{
int  fd, errcnt, work;
char nameout[1024];

/* Get the modified name */
PIiConvertName( name, nameout );

/* Create directories as necessary */
/*SYMakeAllDirs( nameout, fmode ); */

/* Open the file */
fd = nslopen( nameout, mode, fmode );

/* Must let all processors know if the file was successfully opened, 
   and how to identify a message for the file */

/* Check that everyone succeeded. */
errcnt = fd < 0;
PIgisum( &errcnt, 1, &work, procset );

if (errcnt) {
    if (fd) nslclose( fd );
    fd = 0;
    fprintf( stderr, "Failed to open file %s (%s)\n", name, nameout );
    }

return fd;
}


/* @
   PIUnitreeCreate - Creates a context containing the routines for 
   using Unitree operations.

   Returns:
   Pointer to PIFileOps context.
@ */
PIFileOps *PIUnitreeCreate()
{
PIFileOps *new;

new = NEW(PIFileOps);   CHKPTRN(new);
new->basewriter	   = PIiDefUnitreeWrite;
new->basereader	   = PIiDefUnitreeRead;
new->baselseek	   = PIiDefUnitreeLseek;
new->baseflush	   = PIiDefUnitreeFlush;
new->baseopen	   = PIiDefUnitreeOpen;
new->baseclose	   = PIiDefUnitreeClose;
new->baseopenlocal = PIiDefUnitreeOpenEach;

return new;
}
/* 
   These routines provide for adding access to Unitree only when available
   Return 1 if Unitree available, 0 otherwise.
 */
int PIOAddUnitree()
{
  PIORegisterAll();
  PIORegister( -1,    "unitree",     PIUnitreeCreate );
  return 1;
}
#else
int PIOAddUnitree()
{
return 0;
}
#endif

