#ifndef lint
static char vcid[] = "$Id: outman.c,v 1.3 1995/02/13 16:54:52 gropp Exp $";
#endif

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "tools.h"
#include "doc/doc.h"

static int InVerbatim = 0;  /* ??? */

/* These are used to implement '\tt' and `\em`  */
extern int DoQuoteFormat;
static int EmCnt = 1;
static int TtCnt = 1;

static int InCaption = 0;
/*
  man macros that are available:

  .TH t s c n 
   Title t, Section s, commentary c (e.g., local), and manual name n

  .SH text
   Subhead text

  .SS text
   Sub-Subhead text
 
  .B text
   Bold text

  .I text
   Italics text

  .SM text
   Make text 1 point smaller

  .P 
   Begin paragraph

  .BP in
    Begin paragraph with hanging indent

  .PD v
    Set interparagraph spacing to v vertical spaces
 */

/*
   In order to maximize the flexibility of the formating, the output
   is run through these routines.  This will allow us to generate a 
   LaTeX reference manual (perhaps even LaTeXInfo!) with the same
   program.  

   Naturally, this is under construction
 */

/* Beginning of file output */
void OutputBOFMan( fout, basedir )
FILE *fout;
char *basedir;
{
}

void OutputCharMan( fout, c )
FILE *fout;
char c;
{
if (InCaption) return;	
if (c == '\\') 
    fputs( "\\\\", fout );
else if (c == '`' && DoQuoteFormat) {
    /* Emphasis is `  ` */
    if (EmCnt & 0x1) fputs( "\n.B ", fout );
    else             fputs( "\n", fout );
    EmCnt++;
    }
else if (c == '\'' && DoQuoteFormat) {
    /* Tt is ' ' */
    if (TtCnt & 0x1) fputs( "\n.I ", fout );
    else             fputs( "\n", fout );
    TtCnt++;
    }
else {
    putc( c, fout );
    }
}

void OutputRawCharMan( fout, c )
FILE *fout;
char c;
{
if (InCaption) return;	
putc( c, fout );
}

void OutputStringMan( fout, s )
FILE *fout;
char *s;
{
char *p, c, *rstr = 0;

if (InCaption) return;	
/* look for special characters */
p = s;
while (*p) {
    c = *p;
    rstr = 0;
    if (c == '\\') 
	rstr = "\\\\";
    else if (c == '`' && DoQuoteFormat) {
        /* Emphasis is `  ` */
        if (EmCnt & 0x1) rstr = "\n.B ";
        else             rstr = "\n";
        EmCnt++;
        }
    else if (c == '\'' && DoQuoteFormat) {
        /* Tt is ' ' */
        if (TtCnt & 0x1) rstr = "\n.I ";
        else             rstr = "\n";
        TtCnt++;
        }

    if (rstr) {
        *p = '\0';
        fputs( s, fout );
        fputs( rstr, fout );
	s = p + 1;
        }
    p++;
    }
if (*s) 
    fputs( s, fout );
}

void OutputBlankMan( fout )
FILE *fout;
{
/* This probably needs to be changed */
fputs( " ", fout );
}

void OutputLocationMan( fout, s )
FILE *fout;
char *s;
{
fprintf( fout, "\n.SH LOCATION\n %s\n", s );
}

void OutputTitleMan( fout, name, level, date, heading )
FILE *fout;
char *name, *level, *date, *heading;
{
fprintf( fout, ".TH %s %s \"%s\" \" \" \"%s\"\n.SH NAME\n%s ", 
 	     name, level, date, heading, name );
}

void OutputSectionMan( fout, name )
FILE *fout;
char *name;
{
UpperCase( name );
fprintf( fout, ".SH %s\n", name );
}

void OutputLineBreakMan( lastnl, fout )
int  lastnl;
FILE *fout;
{
if (lastnl) 
    fputs( ".br\n", fout );
else
    fputs( "\n.br\n", fout );
}

void OutputArgBeginMan( fout )
FILE *fout;
{
fputs( ".PD 0\n.TP\n.B ", fout );
}

void OutputArgDefnMan( fin, fout )
FILE *fin, *fout;
{
int c;

SkipWhite( fin );
fputs( "\n", fout );
}

void OutputArgEndMan( fout )
FILE *fout;
{
fputs( ".PD 1\n", fout );
}

void OutputVerbatimBeginMan( fout )
FILE *fout;
{
fputs( ".nf\n", fout );
InVerbatim = 1;
}

void OutputVerbatimEndMan( fout )
FILE *fout;
{
fputs( ".fi\n", fout );
InVerbatim = 0;
}

void OutputEndPageMan( fout )
FILE *fout;
{
}

void OutputEndParMan( fout )
FILE *fout;
{
fputs( "\n", fout );	
}

void OutputStartEmMan( fout )
FILE *fout;
{
fputs( "\n.B ", fout );	
}

void OutputEndEmMan( fout )
FILE *fout;
{
fputs( "\n", fout );	
}

void OutputStartTtMan( fout )
FILE *fout;
{
fputs( "\n.I ", fout );
}

void OutputEndTtMan( fout )
FILE *fout;
{
fputs( "\n", fout );
}

void OutputPictureMan( fout, name )
FILE *fout;
char *name;
{
}

void OutputStartCaptionMan( fout )
FILE *fout;
{
InCaption++;
}

void OutputEndCaptionMan( fout )
FILE *fout;
{
InCaption--;	
}

DocOutput *CreateOutputMan( )
{
DocOutput *new;

new = NEW( DocOutput );
new->outbof           = OutputBOFMan;
new->outchar          = OutputCharMan;
new->outraw           = OutputRawCharMan;
new->outstring        = OutputStringMan;
new->outblank         = OutputBlankMan;
new->outlocation      = OutputLocationMan;
new->outtitle         = OutputTitleMan;
new->outsection       = OutputSectionMan;
new->outlinebreak     = OutputLineBreakMan;
new->outargbegin      = OutputArgBeginMan;
new->outargdefn       = OutputArgDefnMan;
new->outargend        = OutputArgEndMan;
new->outverbatimbegin = OutputVerbatimBeginMan;
new->outverbatimend   = OutputVerbatimEndMan;
new->outendpage       = OutputEndPageMan;
new->outendpar        = OutputEndParMan;
new->outstartem       = OutputStartEmMan;
new->outendem         = OutputEndEmMan;
new->outstarttt       = OutputStartTtMan;
new->outendtt         = OutputEndTtMan;
new->outpicture       = OutputPictureMan;
new->outstartcaption  = OutputStartCaptionMan;
new->outendcaption    = OutputEndCaptionMan;

return new;
}

