#ifndef lint
static char vcid[] = "$Id: outtex.c,v 1.3 1995/02/13 16:54:55 gropp Exp $";
#endif

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "tools.h"
#include "doc/doc.h"

/*
    This file contains routines to generate LaTeX output for the structured
    documentation commands
 */

static int InVerbatim = 0; /* ????? */

/* These are used to implement '\tt' and `\em`  */
extern int DoQuoteFormat;
static int EmCnt = 1;
static int TtCnt = 1;

/* Beginning of file output */
void OutputBOFLaTeX( fout, basedir )
FILE *fout;
char *basedir;
{
}

void OutputCharLaTeX( fout, c )
FILE *fout;
char c;
{
if (c == '\\') 
    fputs( "{\\tt \\char`\\\\}", fout );
else {
    if (!InVerbatim) {
	if (c == '<')
	    fputs( "$<$", fout );
	else if (c == '>') 
	    fputs( "$>$", fout );
	else if (c == '=')
	    fputs( "$=$", fout );
	else if (c == '^')
	    fputs( "{\\tt \\char`\\^}", fout );
	else if (c == '_')
	    fputs( "{\\tt \\char`\\_}", fout );
	else if (c == '#')
	    fputs( "{\\tt \\char`\\#}", fout );
	else if (c == '&')
	    fputs( "\\&", fout );
	else if (c == '$')
	    fputs( "\\$", fout );
	else if (c == '%')
	    fputs( "\\%", fout );
	else if (c == '|')
	    fputs( "{\\tt \\char`\\|}", fout );
	else if (c == '{')
	    fputs( "\\{", fout );
	else if (c == '}')
	    fputs( "\\}", fout );
        else if (c == '`' && DoQuoteFormat) {
    	    /* Emphasis is `  ` */
    	    if (EmCnt & 0x1) fputs( "{\\em ", fout );
    	    else             fputs( "}", fout );
    	    EmCnt++;
            }
        else if (c == '\'' && DoQuoteFormat) {
    	    /* Tt is ' ' */
    	    if (TtCnt & 0x1) fputs( "{\\tt ", fout );
    	    else             fputs( "}", fout );
    	    TtCnt++;
	    }
	else
	    putc( c, fout );
	}
    else
	putc( c, fout );
    }
}

void OutputRawCharLaTeX( fout, c )
FILE *fout;
char c;
{
putc( c, fout );
}

void OutputStringLaTeX( fout, s )
FILE *fout;
char *s;
{
char *p, c, *rstr = 0;

/* look for special characters */
p = s;
while (*p) {
    c = *p;
    rstr = 0;
    if (c == '\\') 
	rstr = "{\\tt \\char`\\\\}";
    if (!InVerbatim) {
	if (c == '<')
	    rstr = "$<$";
	else if (c == '>') 
	    rstr = "$>$";
	else if (c == '=')
	    rstr = "$=$";
	else if (c == '^')
	    rstr = "{\\tt \\char`\\^}";
	else if (c == '_')
	    rstr = "{\\tt \\char`\\_}";
	else if (c == '#')
	    rstr = "{\\tt \\char`\\#}";
	else if (c == '&')
	    rstr = "\\&";
	else if (c == '%')
	    rstr = "\\%";
	else if (c == '|')
	    rstr = "{\\tt \\char`\\|}";
	else if (c == '$')
	    rstr = "\\$";
	else if (c == '{')
	    rstr = "\\{";
	else if (c == '}')
	    rstr = "\\}";
        else if (c == '`' && DoQuoteFormat) {
    	    /* Emphasis is `  ` */
    	    if (EmCnt & 0x1) rstr = "{\\em ";
    	    else             rstr = "}";
    	    EmCnt++;
            }
        else if (c == '\'' && DoQuoteFormat) {
    	    /* Tt is ' ' */
    	    if (TtCnt & 0x1) rstr = "{\\tt ";
    	    else             rstr = "}";
    	    TtCnt++;
	    }
	}

    if (rstr) {
        *p = '\0';
        fputs( s, fout );
        fputs( rstr, fout );
	s = p + 1;
        }
    p++;
    }
if (*s) 
    fputs( s, fout );
}

void OutputBlankLaTeX( fout )
FILE *fout;
{
fputs( "\\ ", fout );
}

void OutputLocationLaTeX( fout, s )
FILE *fout;
char *s;
{
fprintf( fout, "\n\\location{%s}\n", s );
}

void OutputTitleLaTeX( fout, name, level, date, heading )
FILE *fout;
char *name, *level, *date, *heading;
{
char bname[256];
strcpy( bname, name );
/* We have to be prepared to correct the name if it contains a
   reserved character */
fputs( "\\mantitle{", fout );
OutputStringLaTeX( fout, name );
fprintf( fout, "}{%s}{%s}\n\\manname{", level, date, name );
OutputStringLaTeX( fout, bname );
fputs( "}\n", fout );
}

void OutputSectionLaTeX( fout, name )
FILE *fout;
char *name;
{
fprintf( fout, "\\subhead{%s}\n", name );
}

void OutputLineBreakLaTeX( lastnl, fout )
int  lastnl;
FILE *fout;
{
fputs( "\\nextline\n", fout );
}

void OutputArgBeginLaTeX( fout )
FILE *fout;
{
fputs( "\\startarg{", fout );
}

void OutputArgDefnLaTeX( fin, fout )
FILE *fin, *fout;
{
int c;

SkipWhite( fin );
fputs( "}{", fout );
c = getc( fin );
if (c != '-') 
    ungetc( (char)c, fin );
else
    SkipWhite( fin );
}

void OutputArgEndLaTeX( fout )
FILE *fout;
{
fputs( "}\n", fout );
}

void OutputVerbatimBeginLaTeX( fout )
FILE *fout;
{
fputs( "\\startvb\\begin{verbatim}\n", fout );
InVerbatim = 1;
}

void OutputVerbatimEndLaTeX( fout )
FILE *fout;
{
fputs( "\\end{verbatim}\n\\endvb", fout );
InVerbatim = 0;
}

void OutputEndPageLaTeX( fout )
FILE *fout;
{
fputs( "\\endmanpage\n", fout );
}

void OutputEndParLaTeX( fout )
FILE *fout;
{
fputs( "\n", fout );	
}

void OutputStartEmTeX( fout )
FILE *fout;
{
fputs( "{\\em ", fout );
}

void OutputEndEmTeX( fout )
FILE *fout;
{
fputs( "}", fout );
}

void OutputStartTtTeX( fout )
FILE *fout;
{
fputs( "{\\tt ", fout );
}

void OutputEndTtTeX( fout )
FILE *fout;
{
fputs( "}", fout );
}

void OutputPictureTeX( fout, name )
FILE *fout;
char *name;
{
fprintf( fout, "\\centerline{\\epsfbox{%s}}", name );
}

void OutputStartCaptionTeX( fout )
FILE *fout;
{
}

void OutputEndCaptionTeX( fout )
FILE *fout;
{
}

DocOutput *CreateOutputLaTeX( )
{
DocOutput *new;

new = NEW( DocOutput );
new->outbof           = OutputBOFLaTeX;
new->outchar          = OutputCharLaTeX;
new->outraw           = OutputRawCharLaTeX;
new->outstring        = OutputStringLaTeX;
new->outblank         = OutputBlankLaTeX;
new->outlocation      = OutputLocationLaTeX;
new->outtitle         = OutputTitleLaTeX;
new->outsection       = OutputSectionLaTeX;
new->outlinebreak     = OutputLineBreakLaTeX;
new->outargbegin      = OutputArgBeginLaTeX;
new->outargdefn       = OutputArgDefnLaTeX;
new->outargend        = OutputArgEndLaTeX;
new->outverbatimbegin = OutputVerbatimBeginLaTeX;
new->outverbatimend   = OutputVerbatimEndLaTeX;
new->outendpage       = OutputEndPageLaTeX;
new->outendpar        = OutputEndParLaTeX;
new->outstartem       = OutputStartEmTeX;
new->outendem         = OutputEndEmTeX;
new->outstarttt       = OutputStartTtTeX;
new->outendtt         = OutputEndTtTeX;
new->outpicture       = OutputPictureTeX;
new->outstartcaption  = OutputStartCaptionTeX;
new->outendcaption    = OutputEndCaptionTeX;

return new;
}
