/*
     Include file for the matrix component of PETSc
*/
#ifndef __MAT 
#define __MAT
#include "packages.h"

#if !defined(IS_COMPONENT)
  Index set package required for matrix package
#endif
#if !defined(VEC_COMPONENT)
  Vector package required for matrix package
#endif
#include "vec.h"

typedef struct _Mat*           Mat;
typedef double*                MatScalar;
typedef struct _MatScatterCtx* MatScatterCtx;

/* Routines */
#ifdef ANSI_ARG
#undef ANSI_ARG
#endif
#ifdef __STDC__
#define ANSI_ARGS(a) a
#else
#define ANSI_ARGS(a) ()
#endif

extern Mat MatCreateSequentialDense   ANSI_ARGS((int,int));
extern Mat MatCreateSequentialAIJ     ANSI_ARGS((int,int));
  
extern int  MatValidMatrix            ANSI_ARGS((Mat));

extern void MatInsertValues      ANSI_ARGS((Mat,MatScalar,int,int*,int,int*));
extern void MatAddValues         ANSI_ARGS((Mat,MatScalar,int,int*,int,int*));
extern void MatBeginAssembly     ANSI_ARGS((Mat));
extern void MatEndAssembly       ANSI_ARGS((Mat));

extern void MatGetValues         ANSI_ARGS((Mat,MatScalar,int,int*,int,int*));
extern void MatGetRow            ANSI_ARGS((Mat,int,int *,int **,MatScalar *));
extern void MatRestoreRow        ANSI_ARGS((Mat,int,int *,int **,MatScalar *));
extern void MatGetCol            ANSI_ARGS((Mat,int,int *,int **,MatScalar *));
extern void MatRestoreCol        ANSI_ARGS((Mat,int,int *,int **,MatScalar *));

extern void MatMult         ANSI_ARGS((Mat,Vec,Vec));
extern void MatMultAdd      ANSI_ARGS((Mat,Vec,Vec,Vec));
extern void MatMultTran     ANSI_ARGS((Mat,Vec,Vec));
extern void MatMultTranAdd  ANSI_ARGS((Mat,Vec,Vec,Vec));

extern void MatLUFactor           ANSI_ARGS((Mat));
extern void MatCholeskyFactor     ANSI_ARGS((Mat));
extern void MatSolve              ANSI_ARGS((Mat,Vec,Vec));
extern void MatSolveAdd           ANSI_ARGS((Mat,Vec,Vec,Vec));
extern void MatSolveTran          ANSI_ARGS((Mat,Vec,Vec));
extern void MatSolveTranAdd       ANSI_ARGS((Mat,Vec,Vec,Vec));

extern void MatRelax         ANSI_ARGS((Mat,Vec,double,int,Vec));
extern void MatRelaxForward  ANSI_ARGS((Mat,Vec,double,Vec));
extern void MatRelaxBackward ANSI_ARGS((Mat,Vec,double,Vec));

extern Mat MatCopy          ANSI_ARGS((Mat));
extern void MatView         ANSI_ARGS((Mat,void*));
extern void MatPrintMatlab  ANSI_ARGS((Mat,FILE*,char *));
extern int  MatNonZeros     ANSI_ARGS((Mat));
extern int  MatMemoryUsed   ANSI_ARGS((Mat));
extern void MatGetDiagonal  ANSI_ARGS((Mat,Vec *));
extern void MatTranspose    ANSI_ARGS((Mat));
extern void MatScale        ANSI_ARGS((Mat,Vec,Vec));
extern void MatShrink       ANSI_ARGS((Mat,int,int*,int,int*));
extern int  MatEqual        ANSI_ARGS((Mat,Mat));
extern Mat  MatGetSubMatrix ANSI_ARGS((Mat,IS,IS));
extern void MatSetSubMatrix ANSI_ARGS((Mat,Mat,IS,IS));
extern void MatReOrder      ANSI_ARGS((Mat,IS,IS));

#define NORM_1         1
#define NORM_2         2
#define NORM_FROBENIUS 3
#define NORM_INFINITY  4
extern void MatNorm         ANSI_ARGS((Mat,int,double *));

extern void MatDestroy      ANSI_ARGS((Mat));
#endif


