/* 
   This defines the abstract vector component. These are patterned
   after the Level-1 Blas, but with some additions that have proved
   useful. These include routines to allocate and free vectors.

   Note that the routines that are normally thought of as returning a
   value (e.g., dot, norm) return their value through an argument.
   This allows these routines to be used with other datatype, such
   as float and dcomplex.

   All vectors should be declared as a Vec. All vector routines begin
   with Vec.
   

 */

#ifndef __VEC 
#define __VEC

#include "packages.h"

#if !defined(IS_COMPONENT)
  Index Set package must be installed for Vector pacakge
#endif
#include "index.h"

typedef struct _Vec*           Vec;
typedef struct _VecScatterCtx* VecScatterCtx;
typedef double*                VecScalar;

/* Routines */
#ifdef ANSI_ARG
#undef ANSI_ARG
#endif
#ifdef __STDC__
#define ANSI_ARGS(a) a
#else
#define ANSI_ARGS(a) ()
#endif

extern Vec VecCreateSequential          ANSI_ARGS((int));  
extern Vec VecCreateSequentialBLAS      ANSI_ARGS((int)); 
extern Vec VecCreateComplexSequential   ANSI_ARGS((int));

#if defined(stderr) 
extern Vec VecCreateInitialVector   ANSI_ARGS((int,int *,char **,FILE*)); 
#else
extern Vec VecCreateInitialVector   ANSI_ARGS((int,int *,char **,void*)); 
#endif

#if defined(COMM_COMPONENT) 
extern Vec VecCreateChameleon            ANSI_ARGS((ProcSet *,int,int));  
extern Vec VecCreateChameleonBLAS        ANSI_ARGS((ProcSet *,int,int));  
extern Vec VecCreateComplexChameleon     ANSI_ARGS((ProcSet *,int,int));
#endif  

#if defined(MPI_COMPONENT)
extern Vec VecCreateMPI            ANSI_ARGS((void *,int,int));  
extern Vec VecCreateMPIBLAS        ANSI_ARGS((void *,int,int));  
extern Vec VecCreateComplexMPI     ANSI_ARGS((void *,int,int));  
#endif

extern void VecDot           ANSI_ARGS((Vec, Vec, VecScalar));
extern void VecTDot          ANSI_ARGS((Vec, Vec, VecScalar));  
extern void VecMDot          ANSI_ARGS((int,      Vec ,Vec*,VecScalar));
extern void VecMTDot         ANSI_ARGS((int,      Vec ,Vec*,VecScalar)); 
extern void VecNorm          ANSI_ARGS((Vec, VecScalar));
extern void VecASum          ANSI_ARGS((Vec, VecScalar));
extern void VecMax           ANSI_ARGS((Vec, int *,    VecScalar));
extern void VecScale         ANSI_ARGS((VecScalar, Vec));    
extern void VecCopy          ANSI_ARGS((Vec, Vec));        
extern void VecSet           ANSI_ARGS((VeScalar, Vec));    
extern void VecSwap          ANSI_ARGS((Vec, Vec));
extern void VecAXPY          ANSI_ARGS((VecScalar, Vec, Vec));  
extern void VecMAXPY         ANSI_ARGS((int,      VecScalar, Vec ,Vec*));
extern void VecAYPX          ANSI_ARGS((VecScalar, Vec, Vec));
extern void VecWAXPY         ANSI_ARGS((VecScalar, Vec, Vec, Vec));
extern void VecPMult         ANSI_ARGS((Vec, Vec, Vec));    
extern void VecPDiv          ANSI_ARGS((Vec, Vec, Vec));    
extern Vec  VecCreate        ANSI_ARGS((Vec));          
extern void VecDestroy       ANSI_ARGS((Vec));        
extern Vec *VecGetVecs       ANSI_ARGS((Vec, int));         
extern void VecFreeVecs      ANSI_ARGS((Vec*,int)); 

extern void VecAddValues     ANSI_ARGS((Vec, VecScalar, int, int *));
extern void VecInsertValues  ANSI_ARGS((Vec, VecScalar, int, int *));
extern void VecBeginAssembly ANSI_ARGS((Vec));
extern void VecEndAssembly   ANSI_ARGS((Vec));

extern void VecScatterBegin  ANSI_ARGS((Vec,IS,Vec,IS,VecScatterCtx *));
extern void VecScatterEnd    ANSI_ARGS((Vec,IS,Vec,IS,VecScatterCtx *)); 

extern void VecScatterAddBegin ANSI_ARGS((Vec,IS,Vec,IS,VecScatterCtx *));
extern void VecScatterAddEnd   ANSI_ARGS((Vec,IS,Vec,IS,VecScatterCtx *));  

extern void *VecGetArray     ANSI_ARGS((Vec));
extern int  VecValidVector   ANSI_ARGS((Vec));
extern void VecView          ANSI_ARGS((Vec,void *));

extern int  VecGetSize       ANSI_ARGS((Vec));
extern int  VecGetLocalSize  ANSI_ARGS((Vec));

#endif


