.TH InstallingTools r "1/3/1995" " " "PETSc"
.SH NAME
InstallingTools \- Procedure for installing the PETSc libraries

.SH BASIC INSTALLATION INFORMATION

All pathnames in this document are relative to the root directory
for the entire PETSc library, given by the environmental variable
ITOOLSDIR (which is usually something like "tools" or "tools.core").
The root directory of PETSc is automatically set during the
installation process.

.br
  *******************************************************************
.br
.br
     MAJOR CHANGES FROM VERSION 1.1 !!!!!!!!!!!!!!!
.br
.br
  *******************************************************************

With the 1.2 release of PETSc and the component packages (Chameleon,
SLES, and KSP), we are providing an alternate method of installation
that uses the widely available GNU configure approach.  A shell script,
configure, is located in the root PETSc directory.  This script will
generate makefiles throughout all PETSc directories that are customized
according to architecture, level of optimization, and site-specific
auxiliary libraries and options.  These makefiles, which are named
"Makefile", are generated in each PETSc directory from the corresponding
file "Makefile.in".  A simple installation can be produced by doing
.br
.br
      configure -bopt=g 
.br
      make 
.br
Currently, you may need to do
.br
.br
      make ranlib_all
.br
after the make has completed.  We are working on eliminating this step.

Chameleon or BlockComm users may need to specify a message-passing
transport layer.  For example, to use p4, do
.br
.br
      configure -bopt=g -comm=p4
.br
      make 
.br
To cross-compile for Intel NX systems, use
.br
.br
      configure -bopt=g -arch=intelnx
.br
      make 
.br
(Use -arch=paragon for Intel Paragons and -arch=inteldelta for the
Intel Delta.)

To get the full range of functionallity from PETSc, you should build
all three "bopt" versions.  "bopt" stands for Base Optimization.  Three
values are supported: -bopt=g (debugging), -bopt=O (optimized), and
-bopt=Opg (profiling for gprof).  When building several versions of
PETSc, you need to clean the directories with the target clean_petsc
prior to each build.  For example,
.br
.br
      configure -bopt=g 
.br
      make 
.br
      make clean
.br
      configure -bopt=O
.br
      make 
.br
      make clean
.br
      configure -bopt=Opg 
.br
      make 
.br
When using this approach, you cannot use arguments on the command line
to make to select the different versions as you could under the previous
system.  However, you should be able to use them with the 'makefile.sv'
versions of the make files.

When building a communication version (e.g., -comm=p4 or -comm=pvm3)
after building PETSc, use the petsc_comm target
.br
.br
      configure -bopt=g -comm=p4
.br
      make petsc_comm
.br

You may also need to specify additional options when building a
communication version, if PETSc/Chameleon does not know about your
system.  These options are
.br
.br
      -comminc=PATH             Include path
.br
      -commopt="options"        Special options
.br
      -commlib=LIBRARY          Communications library
.br
For example, to use MPI, you may need
.br
.br
      -comminc=/usr/local/mpich/include 
.br
      -commlib=/usr/local/mpich/lib/sun4/ch_p4/libmpi.a 
.br
(This assumes the ANL/MSU MPICH implementation, installed in
/usr/local/mpich, for Sun4 with the ch_p4 MPICH device.)

The actual makefiles (both Makefile.in and makefile.sv) are created from
makefile.base in each directory.  Shell scripts in tools.core/cfg
(not currently distributed) add rules and targets to each makefile.base;
the script cfg/updatemake will update all of the makefiles in a directory
tree.  These scripts and the format of makefile.base are still under
development; suggestions and comments are welcome.

Some features are no longer built by default.  For example, the routines
that provide an interface from other message-passing systems to Chameleon
(nx2comm$(COMM).o etc) are only created if you do
.br
.br
      make chameleon_iface
.br

.SH USING OTHER LIBRARIES

The LAPACK and BLAS libraries are no longer included in the PETSc
distribution.  Thus, the previous configure examples build only the
portions of PETSc that do not require these libraries.  The user can
specify the LAPACK and/or BLAS libraries during the configure portion
of installation, as shown by the example
.br
.br
      configure -bopt=g -lapack=<lpath> -blas=<bpath> 
.br
where <lpath> and <bpath> should specify particular files; for example,
.br
.br
       -blas=/usr/local/blas/lib/blas.a
.br
       -lapack=/usr/local/lapack/lib/lapack.a
.br

PETSc provides a high-level interface to BlockSolve, which is a
scalable, parallel library for solving large, sparse, symmetric
systems of linear equations.  Since BlockSolve is not included in
the PETSc distribution, the previous configure examples build only
the portions of PETSc that do not require BlockSolve.  The user can
specify the BlockSolve library during the configure portion of
installation, as shown by the example
.br
.br
      configure -bslib=<bspath> -bsinc=<bsincpath>
.br
where <bspath> should specify a particular library file, and
<bsincpath> should specify the directory containing the BlockSolve
include files.  For example,
.br
.br
      -bslib=/home/curfman/block_solve/bslib.a
.br
      -bsinc=/home/curfman/block_solve
.br

Note that users should augment all of the configure examples to
incorporate the -lapack, -blas, -bslib, and -bsinc options if
using these libraries.

When using command line options with configure (as illustrated above),
do NOT place spaces on either side of the "=" signs.  For example, use
.br
      configure -bopt=g -blas=<bpath> 
.br
instead of
.br
.br
      configure -bopt = g  -blas = <bpath> 

.SH FORTRAN INTERFACE

The Fortran interface has changed from previous versions.  The library
containing the Fortran interface routines is now placed into the same
directory that contains the other libraries.  The source files are
placed in the same directory as the files that they are interfacing to.
In addition, the Fortran interface libraries have moved; they are no
longer in .../tools.core/fort/$(ARCH)/fort$(COMM).a . Instead,
they are in $(LDIR)/libpetscfort$(COMM).a (where $(LDIR) is the usual
PETSc library directory, i.e., .../tools.core/libs/libs$(BOPT)/$(ARCH)).

To build the Fortran interface libraries, use
.br
.br
      make fort_iface
.br
(The default target does this for you).
Because the interface libraries can be big, particularly when built with
BOPT=g, you may wish to build only BOPT=O versions and then provide
links to the others.  For example,
.br
.br
      configure -bopt=O -comm=p4
.br
      make fort_iface
.br
      make fort_links
.br
The target "fort_all" will do this for you, searching for the available
transport (comm) layers.

.SH BUILDING DOCUMENTATION

Documentation for the entire package can be generated once the system
has been configured.  As described above, configuration can be done
with a command such as
.br
.br
      configure -bopt=g 
.br
Then, the following command (which must be executed from the root of
the PETSc directory, $TOOLSDIR) will generate the PETSc man pages in
$TOOLSDIR/man.
.br
.br
      make manpages
.br
This command will also build the "whatis" file (used for keyword
searches with "man -k").  In addition, this command will build
summary files in the directory $TOOLSDIR/docs, which are LaTeX
files that may be used to generate a table of routine names, calling
sequences, and short descriptions.  Executing "make functions.dvi"
from $TOOLSDIR/docs will build the dvi file for this listing;
"make print" will print a copy (assuming that you have dvips.
Otherwise, use your dvi-to-ps converter.)

The script $TOOLSDIR/bin/toolman under X-windows will then display
an interface to all the manual pages for PETSc.  You can pass toolman
additional arguments; for example,
.br
.br
  $TOOLSDIR/bin/toolman -pagesize 630x760 
.br
will change the size of the manual page.

.SH BUILDING EXAMPLES

The command
.br
.br
      make example_progs 
.br
in any directory (including the top-level directory) will build the
example programs for that part of PETSc.  The command
.br
.br
      make run_examples
.br
will run the example programs (it also builds them if necessary).
WARNING-parallel programs will run only
if they can be started without any special "loading" command.  For
example, the Intel Delta requires that programs run with "mexec"; these
programs can not be run with "run_examples".

.SH GENERATING CUSTOM MAKEFILES

The GNU configure process produces the file "config.status" in
the base tools directory.  This file contains user-specified
information about the architecture, level of optimization,
communication, etc., and can then be used to generate custom
makefiles for user applications.  See the man page ToolsMake
for details.

.SH ADVANCED TOPIC
Using tools.core/bin/install

This approach should still work.  Follow the directions in the
man section InstallingTools.Old.  Some people have had no trouble
with this approach; some have had a lot of trouble.  Note that the
Fortran interface builder has been turned OFF by default; you should
try to use the configure mechanism instead.

.SH PROBLEMS?

I expect that this new system will have some problems initially;
please send them to me at gropp@mcs.anl.gov so that I can fix them.
Please send me any output as well as the output of
.br
.br
      uname -a
.br
(This is a program that provides information on the system.)
Thanks in advance for your patience in working with this new system.


.SH LOCATION
 readme
