.TH Overview r "1/3/1995" " " "PETSc"
.SH NAME
Overview \- Goals and contents of PETSc.

PETSc (Portable, Extensible Tools for Scientific computing) is a package
that provides a flexible and uniform framework for methods for solving
some important problems in scientific computing.  This section describes
some of the concepts and organization behind these routines and the
contents of the package.  Since subsets of this package are available,
you may not have the entire package.

.SH CONCEPTS

The PETSc package uses object-oriented design to provide a consistent
and extensible interface to numerical methods and algorithms.  This
approach allows, for example, an application to keep its own data
structures and to use, with little or no code modification, a wide
variety of methods.

Many of the routines in PETSc are arranged around a "context"; this is
a way to hold all of the data that is relevant for describing both the
problem to be solved and method to solve it.  The usual approach is
.br
.br
 ctx = CreateNewContext( method, problem )
.br
 SetOption( ctx, option, value )
.br
 SetOption( ctx, option2, value2 )
.br
 ...
.br
 SolveProblem( ctx )
.br
 FreeContext( ctx )
.br
(The routine names here are generic; each part of PETSc uses its own
set of routines.)

.SH CONTENTS

This list gives the contents of PETSc by file directory, organized into
some broad categories.
.br
.br
.br
 Parallel Processing
.PD 0
.TP
.B blog	
- Event logging.  Used by comm and -event options
.PD 1
.PD 0
.TP
.B comm 
- Chameleon message passing system
.PD 1
.PD 0
.TP
.B blkcm 
- BlockComm package for sending blocks of data
.PD 1
.br
.br
.br
 Linear Systems
.PD 0
.TP
.B sparse 
- General sparse code plus routines for some popular formats
.PD 1
.PD 0
.TP
.B solvers 
- SLES (Simplified Linear Equation Solvers) package
.PD 1
.PD 0
.TP
.B iter 
- Generalized iterative accelerators.  May be used on parallel 
computers.
.PD 1
.PD 0
.TP
.B ilu 
- Some specialized incomplete factorization routines
.PD 1
.PD 0
.TP
.B fd 
- Finite differences in "sparse" format
.PD 1
.br
.br
.br
 Nonlinear Systems
.PD 0
.TP
.B nonlin 
- Routines for solving systems of nonlinear equations and
unconstrained minimization problems (SNES and SUMS)
.PD 1
.br
.br
.br
 Support and Miscellaneous
.PD 0
.TP
.B inline 
- Macros for inlining popular operations.  Should ONLY be used
by experts
.PD 1
.PD 0
.TP
.B set 
- Simple index set operations; used in SLES and BlockComm
.PD 1
.PD 0
.TP
.B vectors 
- Generalized vectors.  Used in SLES, iter, and nonlin
.PD 1
.PD 0
.TP
.B system 
- System support routines (space allocation, timers, file 
management)
.PD 1
.PD 0
.TP
.B c2fort 
- Contains program to construct Fortran interfaces
.PD 1
.PD 0
.TP
.B lint 
- Contains program to construct lint libraries, and the libraries
themselves
.PD 1
.br
  $
.br
 Graphics
.PD 0
.TP
.B xtools 
- X11 Window System routines (xlib level)
.PD 1
.br
.br
.br
 Documentation
.PD 0
.TP
.B ref 
- LaTeX versions of man pages
.PD 1
.PD 0
.TP
.B man 
- man pages
.PD 1
.PD 0
.TP
.B doc 
- Contains programs to produce man pages
.PD 1
.PD 0
.TP
.B docs 
- Documentation on PETSc
.PD 1
.PD 0
.TP
.B docs/tutorial 
- Tutorials on SLES (solvers.tex) and Chameleon (parallel.tex)
.PD 1
.PD 0
.TP
.B docs/talk 
- Overview of PETSc (in Slitex)
.PD 1
.PD 0
.TP
.B tex 
- Some useful TeX macros 
.PD 1
.br
.br
.br
 Scripts and Makefiles
.PD 0
.TP
.B bmake 
- Makefile includes for supported architectures
.PD 1
.PD 0
.TP
.B bin 
- Scripts for installing and using PETSc.
.PD 1


.SH EXAMPLES
Example programs may be found in these directories

.br
 comm/examples       
.br
 comm/examples/angst - Contains programs for performance estimates of 
parallel computers
.br
 sparse/examples     
.br
 solvers/examples    - Contains examples for solving linear systems, 
including some in Fortran.
.br
 iter
.br
 xtools/examples
.br
 xtools/papps/examples
.br
 blkcm/examples
.br
 nonlin/snes/examples
.br
 nonlin/sums/examples


.SH LOCATION
 readme
