.TH ToolsMake r "1/3/1995" " " "PETSc"
.SH NAME
ToolsMake \- Using make with PETSc

.SH INTRODUCTION

PETSc provides a variety of compile-time and link-time options.
In order to simplify the use of PETSc, a standardized makefile format
has been adopted.  These allow easy switching between different
architectures and environments.

.SH DESCRIPTION

The GNU configure installation process in PETSc generates Makefiles
that specify a variety of system dependent variables and options
indicating the compile-time library version to use.  Refer to
InstallingTools for information on the configure installation
process.

Three variables are specified in the Makefiles to indicate
compile-time options.  These are
.PD 0
.TP
.B TOOLARCH=<architecture> 
- where <architecture> should be one of the
supported architectures, such as sun4, intelnx, IRIX, etc.
.PD 1
.PD 0
.TP
.B BOPT=<optimization> 
- where <optimization> should specify the level
of optimization.  Valid values are
.PD 1
.br
                 g (debugging)
.br
                 O (production)
.br
                 Opg (profiling)
.PD 0
.TP
.B COMM=<communication> 
- where <communication> should be the name of
an alternate communication library, such as
.PD 1
.br
                 p4 - p4 communications
.br
                 pvm - (Parallel Virtual Machine)

These variables are then used to specify the base PETSc library
directory,
.br
.br
  LDIR     = $(ITOOLSDIR)/libs/libs$(BOPT)/$(TOOLARCH)
where the base PETSc directory, $(ITOOLSDIR), is set by configure.

Thus, a line to link a program should have the form
.br
.br
  example1: example1.o
.br
            $(CLINKER) -o example1 example1.o \\
.br
            $(LIBS) $(CLIB) $(SLIB) $(FCLIB)
.br
Here the PETSc libraries are denoted by
.br
.br
  LIBS = -L$(LDIR) -l<lib1> -l<lib2> .... -l<libk>
.br
where <lib1>, <lib2>, etc. represent the various PETSc library
modules.  As described in the man page section "Linking", different
PETSc libraries are required depending on the PETSc components
being used.  Refer to the sample Makefile in each component's
examples directory for an appropriate template.

Here, $(CLIB), defined within the Makefile by configure, indicates the
required communication libraries and $(SLIB) gives any required system
libraries.  Non-parallel programs may omit $(CLIB).  $(CLINKER) is the
linker to use for C main programs; use $(FLINKER) for Fortran main
programs.  In most cases these linkers are the same as $(CC) and $(FC),
respectively; however, they may include special options or, on
the CM-5, be different programs entirely.  Here, $(CC) and $(FC) are
symbols defined in the bmake/$(ARCH) file; often these are cc and f77,
but they may be different on some architectures.  For example,
set CC = gcc to use the GNU C compiler.

By default, rules are provided for compiling C and Fortran programs.
These use the values of $(BASEOPT) and $(CFLAGS) (for C) and $(BASEOPTF)
and $(FFLAGS) (for Fortran).  For example, this is the rule for C
programs on a sun4
.br
.br
  .c.o: 
.br
        $(CC) -pipe -c $(CFLAGS) $(BASEOPT) $*.c

The usual definition of CFLAGS is
.br
.br
  CFLAGS   = -I$(ITOOLSDIR) $(OPT)

where $(OPT) is used as a hook for user-defined options (that is, it is
usually null).  For example, to use $(OPT) to pass an option to the
compilation step, use
.br
.br
  make -f Makefile OPT=-DMydefine

Parallel code needs to use

.br
  CFLAGS   = -I$(ITOOLSDIR) $(COPT) $(OPT)

The $(COPT) is essential; this insures that the appropriate version
is built.

If you redefine $(CFLAGS), make sure it includes the
-I$(ITOOLSDIR) $(COPT) values.

The line "LIBNAME = dummy" is not always needed.  If you are building
your own library, say mylib.a, set "LIBNAME = mylib" and use

.br
  SOURCE = mysource.c mysource2.c ...
.br
  SOURCEC = $(SOURCE)
.br
  OBJS   = mysource.o mysource2.o ...

Fortran source files should be specified with "SOURCEF = mysrc.f ..." .

.SH USER-GENERATED MAKEFILES

Users can easily generate their own customized Makefiles for use
with PETSc applications by using the GNU configure process.  This
process involves the following steps.

.PD 0
.TP
.B 1) 
Install PETSc using configure.  This process generates the file
config.status in the main PETSc directory, which can then be used
in conjunction with Makefile.in to generate users' Makefiles.
See the man page InstallingTools for additional information.
.PD 1

.PD 0
.TP
.B 2) 
Copy Makefile.in from the examples directory of the PETSc component 
of interest to a working directory to serve as a template.  For 
example, to develop a Makefile for use with the parallel linear 
solvers, copy $(ITOOLSDIR)/psolvers/examples/Makefile.in
.PD 1

.PD 0
.TP
.B 3) 
Customize this file by retaining the header information but
altering the program examples.  The current format of Makefile.in
contains some superfluous information, although this will not
cause any problems.
.PD 1

.PD 0
.TP
.B 4) 
Set the configure environment variable with
.PD 1
.br
.br
         setenv CONFIG_FILES Makefile 
.br
(This assumes use of the C shell.)

.PD 0
.TP
.B 5) 
Generate a customized Makefile by executing 
.PD 1
.br
.br
         $(TOOLSDIR)/config.status
.br
from your working directory.

.PD 0
.TP
.B 6) 
Unset the configure environment variable with
.PD 1
.br
.br
         unsetenv CONFIG_FILES
.br
(Again, this assumes use of the C shell.)  Note that this
environment variable MUST be unset before rerunning configure
(for example, to set a different level of optimization).

Alternatively, steps 4 and 5 can be done within a subshell, so that
step 6 is unnecessary.

The config.status file that has been generated for a particular set of
options can be saved, and then reused when generating different versions
of Makefiles in the future.  (This saves the effort of rerunnning
configure repeatedly.)  For example, one might desire to create
.br
    config.status.g - debug version
.br
    config.status.O - optimized version

.SH ADVANCED TOPIC - ALTERNATIVE MAKEFILES

The current PETSc release contains alternative makefiles named
"makefile.sv" that correspond to the makefiles of PETSc versions
prior to PETSc 1.2.  These makefiles directly include the
architecture, optimization, and communication information that
is specified within the base make (bmake) directory.  Hence, this
makefile format enables users to employ command-line arguments to
specify $(ARCH), $(BOPT), and $(COMM).

Note that the alternative makefiles (makefile.sv) do NOT incorporate
user-specified information from the GNU configure installation
process.  Thus, to alter the default information within the bmake
directory, users must either manually alter the bmake files
for their particular architecture(s) of interest or override the
options in the makefile.sv files.

.SH LOCATION
 readme
