#ifndef lint
static char vcid[] = "$Id: exitall.c,v 1.4 1995/02/13 16:31:00 gropp Exp $";
#endif

#include <string.h>
#include "tools.h"
#include "system/system.h"
#include <stdio.h>         
#ifndef __MSDOS__

#include <signal.h>          /*I <signal.h> I*/
/* This routine handles signals that usually abort a user's code */
static char *SIGNAME[] = { "Unknown", "HUP", "INT", "QUIT", "ILL",
                           "TRAP",    "ABRT", "EMT", "FPE", "KILL", 
                           "BUS", "SEGV", "SYS", "PIPE", "ALRM",
                           "TERM", "URG", "STOP", "TSTP", "CONT", 
                           "CHLD" }; /* There are others, put we won't 
					report them */


/* 
   In order to allow more elaborate debuggers, such as starting the
   debugger in an xterm or with a -display option, we provide a
   default argument stream in baseargs.  In this case, the arguments are
   baseargs, followed by the process id of the job to stop.
   If baseargs not set, then the given debugger is used.
 */
static int  call_debugger=0;
static char *program=0;
static char *debugger=0;
static char **baseargs=0;
static int  nbaseargs=0;

#if defined(SIGSTOP)
void SY_Start_debugger( )
{
int child, i;

  child = fork(); 
  if (child) { /* I am the parent will run the debugger */
    char  **args, pid[10];
    args = (char **)malloc( (4 + nbaseargs) * sizeof(char *) );
    kill(child,SIGSTOP);
    sprintf(pid,"%d",child); 
    if (nbaseargs > 0) {
	for (i=0; i<nbaseargs; i++) {
	    args[i] = baseargs[i];
	    }
	args[i++] = pid;
	args[i++] = 0;
        debugger  = args[0];
	}
    else {
	args[0] = debugger; args[1] = program; args[2] = pid; args[3] = 0;
    }
    /* fprintf(stderr,"Attaching %s to %s %s\n", debugger, program, pid); */
    if (execvp(debugger, args)  < 0) {
      perror("Unable to start debugger");
      exit(0);
    }
  }
  else { /* I am the child, continue with user code */
    sleep(10);
  }
}
#else
void SY_Start_debugger()
{
fprintf( stderr, "Can not start the debugger on this system\n" );
}
#endif

/*@
   SY_Errors_call_debugger - On an error, print a message and (attempt) to
   start the specified debugger on the program

   Input Parameters:
.  pgm - Name of the program.
.  dbg - Name of the debugger.  If null, use a default (usually dbx)
.  args - arguments to use in generating the debugger.
   This allows things like "xterm -e dbx pgm pid", or 
   "xdbx -geometry +%d+%d pgm pid".  The list should be null terminated.
   (The %d %d format is not yet supported).
@*/
void SY_Errors_call_debugger( pgm, dbg, args )
char *pgm, *dbg, **args;
{
int            i;

if (args) {
    while (args[nbaseargs]) nbaseargs++;
    baseargs = (char **)malloc( (nbaseargs+1) * sizeof(char *) );
    for (i=0; i<=nbaseargs; i++) 
	baseargs[i] = args[i];
    }
else if (!dbg) {
    dbg = "/usr/ucb/dbx";
    }

if (!pgm) {
    fprintf( stderr, 
    "Must specify the program name when setting errors-call-debugger\n" );
    return;
    }
program = (char *)malloc( strlen(pgm) + 1 );
strcpy( program, pgm );
if (dbg) {
    debugger = (char *)malloc( strlen(dbg) + 1 );
    strcpy( debugger, dbg );
    }

/* Need to install... */
call_debugger = 1;
}

char *SYi_Copy_string( str )
char *str;
{
char *new;
new = (char *)malloc( strlen(str) + 1 );
strcpy( new, str );
return new;
}

/* @
@ */
void SY_Errors_call_dbx_in_xterm( pgm, display )
char *pgm, *display;
{
char **args;
int  myid, str_len;

/* By default, we use the name of the root node */
if (!display) {
    extern char *getenv();
    display = getenv( "DISPLAY" );
    if (!display || display[0] == ':') {
	/* Replace display with hostname:0 */
	display = (char *)malloc( 100 );
	SYGetHostName( display, 100 );
	strcat( display, ":0" );
	}
    }

args    = (char **)malloc( 7 * sizeof(char *) );
args[0] = SYi_Copy_string( "xterm" );
args[1] = SYi_Copy_string( "-display" );
args[2] = SYi_Copy_string( display );
args[3] = SYi_Copy_string( "-e" );
args[4] = SYi_Copy_string( "/usr/ucb/dbx" );
args[5] = SYi_Copy_string( pgm );
args[6] = 0;

SY_Errors_call_debugger( pgm, (char *)0, args );
}

/*
  SY_DefaultDBHandler - Default signal handler.

  Input Parameters:
. sig   - signal value
. code,scp,addr - see the signal man page
*/
void SY_DefaultDBHandler( sig, code, scp, addr )
int               sig, code;
struct sigcontext *scp;
char              *addr;
{
static char buf[128];

signal( sig, SIG_DFL );
if (sig >= 0 && sig <= 20) 
    sprintf( buf, "Caught signal %s", SIGNAME[sig] );
else
    strcpy( buf, "Caught signal " );
fprintf( stderr, "%s\n", buf );

SY_Start_debugger( );
}

void SY_Signals_call_debugger()
{
if (!call_debugger) {
    return;
    }
signal( SIGQUIT, (void (*)())SY_DefaultDBHandler );
signal( SIGILL,  (void (*)())SY_DefaultDBHandler );
signal( SIGFPE,  (void (*)())SY_DefaultDBHandler );
signal( SIGBUS,  (void (*)())SY_DefaultDBHandler );
signal( SIGSEGV, (void (*)())SY_DefaultDBHandler );
signal( SIGSYS,  (void (*)())SY_DefaultDBHandler );
}

/*ARGSUSED*/
/*@
  SYDefaultHandler - Default signal handler.

  Input Parameters:
. sig   - signal value
. code,scp,addr - see the signal man page
@*/
void SYDefaultHandler( sig, code, scp, addr )
int               sig, code;
struct sigcontext *scp;
char              *addr;
{
static char buf[128];

signal( sig, SIG_DFL );
if (sig >= 0 && sig <= 20) 
    sprintf( buf, "Caught signal %s", SIGNAME[sig] );
else
    strcpy( buf, "Caught signal " );
/* Should SYExit fail to abort for any reason, resignal the error
   with the default handler */

/* We'd like to provide for a core dump ... */
#ifdef FOO
if (sig == SIGSEGV) {
    kill( getpid(), SIGIOT );
    }
#endif
SYExit( buf, sig );
kill( getpid(), sig );
}

#if defined(HPUX)
#define SIGBUS _SIGBUS
#define SIGSYS _SIGSYS
#endif

/*@
   SYSetDefaultSignals - Set up to catch the usual fatal errors and 
   kill a parallel job.

   Input parameter:
.  routine - routine to call when a signal is received.  This should
             have a form that is compatible with "signal".
@*/
void SYSetDefaultSignals( routine )
void (*routine)();
{
if (routine == 0) routine = SYDefaultHandler;
signal( SIGQUIT, routine );
signal( SIGILL,  routine );
signal( SIGFPE,  routine );
signal( SIGBUS,  routine );
signal( SIGSEGV, routine );
signal( SIGSYS,  routine );
}

#else
void SYSetDefaultSignals( routine )
void (*routine)();
{}
#endif

/* 
   Here are the actual exitall routines.  This allows a parallel version
   to replace the routine with one that exits ALL processes in a 
   parallel process
 */
void SYexitall_default(msg,rc)
char *msg;
int  rc;
{
if (msg) fprintf( stderr, "%s\n", msg );
exit(rc);
}

static void (*SYexitall_routine)() = SYexitall_default;

/*@
    SYexitall - Terminate all processes in an execution (parallel exit).

    Input Parameters:
.   str   - Error message string
.   rc    - return code

    Notes:
    The return code may not be available in some implementations.
@*/
void SYexitall(str,rc)
char *str;
int  rc;
{
if (!SYexitall_routine) exit(rc);
(*SYexitall_routine)(str,rc);
}

/*@
    SYSetExitAll - set the routine to be called by SYexitall (parallel exit)

    Input Parameter:
.   r   - routine.  Should be of the form void r(char *str,int rc)
@*/
void SYSetExitAll(r)
void (*r)();
{
SYexitall_routine = r;
}


/*
   Here is the generic error handler.  
   This is called by the SETERRx macros.
   If "call_debugger" is set, it will activate the debugger.
 */
SY_ErrorHandler( line, file, msg, who )
int  line, who;
char *file, *msg;
{
if (who >= 0) {
    if (!msg)
	fprintf( stderr, "[%d] Line %d in %s\n", who, line, file );
    else
	fprintf( stderr, "[%d] Line %d in %s: %s\n", who, line, file, msg );
    }
else {
    if (!msg)
	fprintf( stderr, "Line %d in %s\n", line, file );
    else
	fprintf( stderr, "Line %d in %s: %s\n", line, file, msg );
    }
#ifndef __MSDOS__    
if (call_debugger)
    SY_Start_debugger();
#endif    
}
